set echo on
column table_name format a30
column column_name format a30
clear screen
remark
remark  Deze demonstratie is een aangepaste versie van een demo op de blog
remark  van Oracle's Optimizer Development Group.
remark
set serveroutput off
alter system flush shared_pool
/
create table t (ename,empno,phone,deptno)
as
 select 'Name ' || to_char(level)
      , level
      , '01234-' || rpad(to_char(level),5,'0')
      , case
        when level <= 100 then trunc((level-1)/10)
        else 10
        end
   from dual
connect by level <= 100000
/
create index t_i1 on t(deptno)
/
begin
  dbms_stats.gather_table_stats
  ( user
  , 't'
  , cascade=>true
  , method_opt=>'FOR COLUMNS DEPTNO SIZE 11'
  , estimate_percent => 100
  );
end;
/
select deptno
     , count(*)
  from t
 group by deptno
 order by deptno
/
select table_name
     , column_name
     , histogram
  from user_tab_cols
 where table_name = 'T'
/
select endpoint_number
     , endpoint_value
  from user_histograms
 where table_name = 'T'
   and column_name = 'DEPTNO'
/
pause
var deptno number
exec :deptno := 9
select /*RWIJK_1*/
       count(*)
     , max(empno)
  from t
 where deptno = :deptno
/
select * from table(dbms_xplan.display_cursor)
/
select child_number
     , executions
     , buffer_gets
     , is_bind_sensitive
     , is_bind_aware
  from v$sql
 where sql_text like 'select /*RWIJK_1%'
/
remark  Dit is wat we hier verwachten:
remark  De eerste keer kijkt de cost based optimizer in de parse-fase naar de
remark  inhoud van de bindingsvariabele, een 9, en optimaliseert de zoekvraag
remark  daarnaar. Je ziet immers een cardinaliteit van 10 in het plan.
remark
remark  Nu hetzelfde met waarde 10, de meeste voorkomende waarde:
pause
exec :deptno := 10
select /*RWIJK_1*/
       count(*)
     , max(empno)
  from t
 where deptno = :deptno
/
select * from table(dbms_xplan.display_cursor)
/
select child_number
     , executions
     , buffer_gets
     , is_bind_sensitive
     , is_bind_aware
  from v$sql
 where sql_text like 'select /*RWIJK_1%'
/
remark  Nog steeds via een index, terwijl we dat nu niet willen.
remark  Peeking leidt de eerste keer tot een goed plan, maar kan een volgende keer
remark  met een andere waarde voor de bindingsvariabele leiden tot een minder plan.
remark  In versies 9 en 10 blijft dit plan altijd gelden.
remark
remark  Maar kijk wat versie 11 doet als we nog een keer opvragen met waarde 10.
remark  Let ook op de IS_BIND_SENSITIVE waarde hierboven.
pause
exec :deptno := 10
select /*RWIJK_1*/
       count(*)
     , max(empno)
  from t
 where deptno = :deptno
/
select * from table(dbms_xplan.display_cursor)
/
select child_number
     , executions
     , buffer_gets
     , is_bind_sensitive
     , is_bind_aware
  from v$sql
 where sql_text like 'select /*RWIJK_1%'
/
remark  Doordat de cursor "BIND_SENSITIVE" is EN het ziet dat het aantal buffer gets
remark  van 3 naar 694 stijgt (getallen kunnen iets afwijken), wordt de cursor nu
remark  "BIND_AWARE". Het plan is dus aangepast en er zijn nu twee kindcursoren:
remark  de initiele en een nieuwe speciaal voor waarde 10.
remark
remark  Vraag nu nogmaals voor waarde 9 uit:
pause
exec :deptno := 9
select /*RWIJK_1*/
       count(*)
     , max(empno)
  from t
 where deptno = :deptno
/
select * from table(dbms_xplan.display_cursor)
/
select child_number
     , executions
     , buffer_gets
     , is_bind_sensitive
     , is_bind_aware
     , is_shareable
  from v$sql
 where sql_text like 'select /*RWIJK_1%'
/
remark  En je ziet een derde kindcursor ontstaan voor de waarde 9.
remark  Merk nog op dat de initiele cursor niet meer gebruikt zal worden.
remark  Deze heeft IS_SHAREABLE op 'N' staan en zal spoedig uitgefaseerd
remark  worden.
pause
exec :deptno := 8
select /*RWIJK_1*/
       count(*)
     , max(empno)
  from t
 where deptno = :deptno
/
select * from table(dbms_xplan.display_cursor)
/
select child_number
     , executions
     , buffer_gets
     , is_bind_sensitive
     , is_bind_aware
     , is_shareable
  from v$sql
 where sql_text like 'select /*RWIJK_1%'
/
set echo off
drop table t purge
/
